Attribute VB_Name = "modCommon"
Option Explicit


'================================================================================
'
' modCommon Module
' ----------------
'
' Created By  : Kevin Wilson
'               http://w3.one.net/~kevinw/VB   ( The VB Zone )
'
' Last Update : June 20, 2000
'
' VB Versions : 5.0 / 6.0
'
' Requires    : NOTHING
'
' Description : This class module is a combination of all of the Windows API's
'               that I use most often.  I kept having to add them every time I
'               created a new project, so I figured I'd save myself some time by
'               creating a reusable module that can be used in any program.
'
' Example Use : ( See Below )
'
'================================================================================


' Public Types / Enumerations
'Public Type POINTAPI
'  X As Long
'  Y As Long
'End Type

'Public Type RECT
'  Left           As Long
'  Top            As Long
'  Right          As Long
'  Bottom         As Long
'End Type

Public Type BROWSEINFO
  hwndOwner      As Long
  pidlRoot       As Long
  pszDisplayName As String
  lpszTitle      As String
  ulFlags        As Long
  lpfnCALLBACK   As Long
  lParam         As Long
  iImage         As Long
End Type

' General Constants
Public Const MAX_PATH = 260

' SetFileAttributes Flag Constants
Public Const FILE_ATTRIBUTE_ARCHIVE = &H20
Public Const FILE_ATTRIBUTE_COMPRESSED = &H800
Public Const FILE_ATTRIBUTE_DIRECTORY = &H10
Public Const FILE_ATTRIBUTE_HIDDEN = &H2
Public Const FILE_ATTRIBUTE_NORMAL = &H80
Public Const FILE_ATTRIBUTE_READONLY = &H1
Public Const FILE_ATTRIBUTE_SYSTEM = &H4
Public Const FILE_ATTRIBUTE_TEMPORARY = &H100

' SetWindowPos Flag Constants
Public Const SWP_FRAMECHANGED = &H20               ' Forces a WM_NCCALCSIZE message to go to the window even if its size does not change.
Public Const SWP_DRAWFRAME = SWP_FRAMECHANGED      ' Draws a frame around the window.
Public Const SWP_HIDEWINDOW = &H80                 ' Hides the window.
Public Const SWP_NOACTIVATE = &H10                 ' Does not activate the window.
Public Const SWP_NOMOVE = &H2                      ' Retains current position (x and y are ignored).
Public Const SWP_NOREDRAW = &H8                    ' Window is not automatically redrawn.
Public Const SWP_NOSIZE = &H1                      ' Retains current size (cx and cy are ignored).
Public Const SWP_NOZORDER = &H4                    ' Retains current position in the window list (hWndInsertAfter is ignored).
Public Const SWP_SHOWWINDOW = &H40                 ' Displays the window.
Public Const SWP_NOOWNERZORDER = &H200             ' Don't do owner Z ordering
Public Const SWP_NOSENDCHANGING = &H400            ' Don't send WM_WINDOWPOSCHANGING
Public Const SWP_FLAGS = SWP_NOMOVE Or SWP_NOSIZE
Public Const HWND_TOP = 0                          ' Place window at the top of the Z-order, the order in which windows are displayed for the given level of the window in the hierarchy
Public Const HWND_BOTTOM = 1                       ' Place window at bottom of the window list
Public Const HWND_TOPMOST = -1                     ' Place window at the top of the list, ahead of any topmost windows
Public Const HWND_NOTOPMOST = -2                   ' Place window at the top of the list, behind any topmost windows

' RemoveMenu Flag Constants
Public Const MF_BYCOMMAND = &H0&                   ' Indicates that the uPosition parameter gives the identifier of the menu item. The MF_BYCOMMAND flag is the default if neither the MF_BYCOMMAND nor MF_BYPOSITION flag is specified.
Public Const MF_BYPOSITION = &H400&                ' Indicates that the uPosition parameter gives the zero-based relative position of the menu item.

' Browse Folder Dialog Flag Constants
Public Const BIF_BROWSEFORCOMPUTER = &H1000        ' * Browsing for Computers
Public Const BIF_BROWSEFORPRINTER = &H2000         ' * Browsing for Printers
Public Const BIF_BROWSEINCLUDEFILES = &H4000       ' * Browsing for Everything
Public Const BIF_RETURNONLYFSDIRS = &H1            ' Only returns file system directories. If the user selects folders that are not part of the file system, the OK button is grayed.
Public Const BIF_DONTGOBELOWDOMAIN = &H2           ' Does not include network folders below the domain level in the tree view control.
Public Const BIF_STATUSTEXT = &H4                  ' Includes a status area in the dialog box. The callback function can set the status text by sending messages to the dialog box.
Public Const BIF_RETURNFSANCESTORS = &H8           ' Only returns file system ancestors. If the user selects anything other than a file system ancestor, the OK button is grayed.
Public Const BIF_VALIDATE = &H20                   ' Insist on valid result (or CANCEL)
Public Const BIF_EDITBOX = &H10                    ' Displays an editable TextBox control in the dialog

' ShowWindow Flag Constants
Public Const SW_HIDE = 0                           ' Hides the window and activates another window.
Public Const SW_MAXIMIZE = 3                       ' Maximizes the specified window.
Public Const SW_MINIMIZE = 6                       ' Minimizes the specified window and activates the next top-level window in the Z order.
Public Const SW_RESTORE = 9                        ' Activates and displays the window. If the window is minimized or maximized, Windows restores it to its original size and position. An application should specify this flag when restoring a minimized window.
Public Const SW_SHOW = 5                           ' Activates the window and displays it in its current size and position.
Public Const SW_SHOWDEFAULT = 10                   ' Sets the show state based on the SW_ flag specified in the STARTUPINFO structure passed to the CreateProcess function by the program that started the application.
Public Const SW_SHOWMAXIMIZED = 3                  ' Activates the window and displays it as a maximized window.
Public Const SW_SHOWMINIMIZED = 2                  ' Activates the window and displays it as a minimized window.
Public Const SW_SHOWMINNOACTIVE = 7                ' Displays the window as a minimized window. The active window remains active.
Public Const SW_SHOWNA = 8                         ' Displays the window in its current state. The active window remains active.
Public Const SW_SHOWNOACTIVATE = 4                 ' Displays a window in its most recent size and position. The active window remains active.
Public Const SW_SHOWNORMAL = 1                     ' Activates and displays a window. If the window is minimized or maximized, Windows restores it to its original size and position. An application should specify this flag when displaying the window for the first time.

' Public Variables
Public ExitingProgram As Boolean

' Public Windows API Declarations
Public Declare Sub Sleep Lib "kernel32.dll" (ByVal dwMilliseconds As Long)
Public Declare Sub CoTaskMemFree Lib "ole32.dll" (ByVal hMem As Long)
Public Declare Function FindWindow Lib "USER32.DLL" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Long
Public Declare Function GetCursorPos Lib "USER32.DLL" (lpPoint As POINTAPI) As Long
Public Declare Function GetPrivateProfileString Lib "kernel32.dll" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Public Declare Function GetSystemMenu Lib "USER32.DLL" (ByVal hWnd As Long, ByVal bRevert As Long) As Long
Public Declare Function GetSystemDirectory Lib "kernel32.dll" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Public Declare Function GetTempPath Lib "kernel32.dll" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Public Declare Function GetWindowsDirectory Lib "kernel32.dll" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Public Declare Function GetWindowRect Lib "USER32" (ByVal hWnd As Long, lpRect As RECT) As Long
Public Declare Function lstrCat Lib "kernel32.dll" Alias "lstrcatA" (ByVal lpString1 As String, ByVal lpString2 As String) As Long
Public Declare Function MoveWindow Lib "USER32" (ByVal hWnd As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long) As Long
Public Declare Function RemoveMenu Lib "USER32.DLL" (ByVal hMenu As Long, ByVal nPosition As Long, ByVal wFlags As Long) As Long
Public Declare Function SetForegroundWindow Lib "USER32.DLL" (ByVal hWnd As Long) As Long
Public Declare Function SetFileAttributes Lib "KERNEL32" Alias "SetFileAttributesA" (ByVal lpFileName As String, ByVal dwFileAttributes As Long) As Long
Public Declare Function SetWindowPos Lib "USER32.DLL" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, ByVal lngLeft As Long, ByVal lngTop As Long, ByVal lngWidth As Long, ByVal lngHeight As Long, ByVal wFlags As Long) As Long
Public Declare Function SHBrowseForFolder Lib "shell32.dll" (lpbi As BROWSEINFO) As Long
Public Declare Function SHGetPathFromIDList Lib "shell32.dll" (ByVal pidList As Long, ByVal lpBuffer As String) As Long
Public Declare Function SHGetIDListFromPath Lib "shell32.dll" Alias "#162" (ByVal szPath As String) As Long
Public Declare Function ShowWindow Lib "USER32.DLL" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Public Declare Function WritePrivateProfileString Lib "kernel32.dll" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long
Private Declare Function SendMessage Lib "USER32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long

'====================================================================================
' AutoComplete
'
' Description:
' ------------
' Searches a specified ComboBox for the closest match to what is being typed. This
' functionality is similar to the nagivation ComboBox in both Netscape and MSIE.
'
' Param                 Description
' ---------------------------------
' Title                 Optional. Specifies the titlebar caption of the window to
'                       activate.  If this is not specified, the ClassName param
'                       must be specified.
' ClassName             Optional. Specifies the class name of the window to actiave.
'                       If this is not specified, the Title param must be specified.
' RestoreWindow         Optional. If set to TRUE, this function will make sure the
'                       specified window is neither minimized nor maximized and is
'                       visible before trying to activate it.
'
' Example Use:
' ------------
'
'  ' Put the following code in the KeyUp event of the ComboBox:
'  Select Case KeyCode
'    Case 32, &H30 To &H6F, Is > &H7F
'      AutoComplete Combo1
'  End Select
'
' Return:
' -------
' NOTHING
'
'====================================================================================
Public Sub AutoComplete(ByRef CboBox As ComboBox)
On Error Resume Next
  
  Dim ReturnValue As Long
  Dim lngPosition As Long

  With CboBox
    lngPosition = Len(.Text)
    If lngPosition <> 0 Then
      ReturnValue = SendMessage(.hWnd, &H14C, -1&, ByVal .Text)
      .ListIndex = ReturnValue
      .SelStart = lngPosition
      .SelLength = Len(.Text) - lngPosition
    End If
  End With

End Sub

'====================================================================================
' App_Activate
'
' Description:
' ------------
' Programatically force the specified window to be active.
'
' Param                 Description
' ---------------------------------
' Title                 Optional. Specifies the titlebar caption of the window to
'                       activate.  If this is not specified, the ClassName param
'                       must be specified.
' ClassName             Optional. Specifies the class name of the window to actiave.
'                       If this is not specified, the Title param must be specified.
' RestoreWindow         Optional. If set to TRUE, this function will make sure the
'                       specified window is neither minimized nor maximized and is
'                       visible before trying to activate it.
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function App_Activate(Optional ByVal Title As String, Optional ByVal ClassName As String = "", Optional ByVal RestoreWindow As Boolean = False) As Boolean
On Error GoTo ErrorTrap
     
     Dim ReturnValue As Long
     
     ' Check for valid parameters, then get the handle of the specified window
20   If ClassName <> "" Then
30     ReturnValue = FindWindow(ClassName, vbNullString)
40     If (ReturnValue = 0 Or ReturnValue = vbNull) And Title = "" Then
50       App_Activate = False
60       Exit Function
70     ElseIf (ReturnValue = 0 Or ReturnValue = vbNull) And Title <> "" Then
80       ReturnValue = FindWindow(vbNullString, Title)
90       If ReturnValue = 0 Or ReturnValue = vbNull Then
100        App_Activate = False
110        Exit Function
120      End If
130    End If
140  ElseIf Title <> "" Then
150    ReturnValue = FindWindow(vbNullString, Title)
160    If ReturnValue = 0 Or ReturnValue = vbNull Then
170      App_Activate = False
180      Exit Function
190    End If
200  Else
210    App_Activate = False
220    Exit Function
230  End If
     
     ' Restore the window first if the user specifies to
240  If RestoreWindow = True Then
250    ShowWindow ReturnValue, SW_SHOW
260    ShowWindow ReturnValue, SW_RESTORE
270  End If
     
280  DoEvents
     
     ' Force the specified window to the forground
290  ReturnValue = SetForegroundWindow(ReturnValue)
300  If ReturnValue = 0 Then
310    App_Activate = False
320  Else
330    App_Activate = True
340  End If
     
350  Exit Function
     
ErrorTrap:
     
360  If Err.Number = 0 Then
370    Resume Next
380  ElseIf Err.Number = 20 Then
390    Resume Next
400  Else
'410    MsgBox "The following error occured while trying to set the following window to the foreground:" & Chr(13) & Chr(13) & "'" & Title & "'" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description & Chr(13) & "Error Line = " & Erl, vbOKOnly + vbExclamation, "  Error Activating Window"
420    Err.Clear
430    App_Activate = False
440    Exit Function
450  End If
    
End Function

'====================================================================================
' BrowseForFolder
'
' Description:
' ------------
' Displays the standard Windows "Browse For Folder" dialog box.
'
' Param                 Description
' ---------------------------------
' ReturnPath            Returns the selected folder.  This returns vbNullString if
'                       the user CANCEL'd the dialog, or an error occured.
' ReturnTitle           Optional. Returns the "Display Name" of the selected folder.
' OwnerHandle           Optional. Defines the owner of the dialog box.
' Prompt                Optional. Specifies the text to display in the dialog
' RootDir               Optional. Specifies the TOP folder of the dialog.  The
'                       default is the Windows Desktop.
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function BrowseForFolder(ByRef ReturnPath As String, Optional ByRef ReturnTitle As String, Optional ByVal OwnerHandle As Long, Optional ByVal Prompt As String, Optional ByVal RootDir As String) As Boolean
On Error Resume Next
  
  ' Declare variables to be used
  Dim ReturnValue As Long
  Dim TempPath As String
  Dim TheBrowseInfo As BROWSEINFO
  Dim DisplayName As String
  Dim ImageIndex As Long
  Dim Flags As Long
  Dim MyAnswer As VbMsgBoxResult
  Dim IDListRoot As Long
  
  ' Create a buffer to recieve the
  DisplayName = String(MAX_PATH, Chr(0))
  
  ' Get the IDList for the specified root
  If RootDir = "" Then
    IDListRoot = 0
  Else
    If Dir(RootDir, vbDirectory) <> "" Then
      IDListRoot = SHGetIDListFromPath(RootDir)
    End If
  End If
  
  ' Initialise variables
  With TheBrowseInfo
    .hwndOwner = OwnerHandle
    .pidlRoot = IDListRoot
    .pszDisplayName = DisplayName
    .lpszTitle = Prompt
    .ulFlags = BIF_RETURNONLYFSDIRS Or BIF_VALIDATE
    .lpfnCALLBACK = 0
    .lParam = 0
    .iImage = ImageIndex
  End With
  
  ' Call the browse for folder API
  ReturnValue = SHBrowseForFolder(TheBrowseInfo)
  
  ' Check if user canceled... and if the user did NOT, get the path selected
  If ReturnValue <> 0 Then
    ' Create a buffer for the path
    TempPath = String(MAX_PATH, 0)
    
    ' Get the path from the pointer
    SHGetPathFromIDList ReturnValue, TempPath
    
    ' Clean up the paths
    If InStr(TempPath, Chr(0)) > 0 Then
      TempPath = Left(TempPath, InStr(TempPath, Chr(0)) - 1)
    End If
    DisplayName = TheBrowseInfo.pszDisplayName
    If InStr(DisplayName, Chr(0)) > 0 Then
      DisplayName = Left(DisplayName, InStr(DisplayName, Chr(0)) - 1)
    End If
    
    ' Free up memory used
    CoTaskMemFree ReturnValue
    
    ' Return the path
    ReturnPath = TempPath
    ReturnTitle = DisplayName
    BrowseForFolder = True
  Else
    ' Clear the return values due to invalid selection / CANCEL
    ReturnPath = ""
    ReturnTitle = ""
    BrowseForFolder = False
  End If
  
End Function

'====================================================================================
' CenterWindow
'
' Description:
' ------------
' Centers the window specified in the middle of the screen.
'
' Param                 Description
' ---------------------------------
' Handle                Optional. Specifies the handle of the window to center. If
'                       this is not specified, the Title or ClassName param must be.
' Title                 Optional. Specifies the titlebar caption of the window to
'                       center.  If this is not specified, the Handle or ClassName
'                       param must be.
' ClassName             Optional. Specifies the class name of the window to center.
'                       If this is not specified, the Handle or Title param must be.
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function CenterWindow(Optional ByVal Handle As Long, Optional ByVal Title As String = "", Optional ByVal ClassName As String = "") As Boolean
On Error GoTo ErrorTrap
  
  Dim TheHandle As Long
  Dim ReturnValue As Long
  Dim W_Top As Integer
  Dim W_Left As Integer
  Dim W_Width As Integer
  Dim W_Height As Integer
  Dim rct As RECT
  
  ' Check for valid parameters, then get the handle of the specified window
  If ClassName = "" And Title = "" And Handle = 0 Then
    Exit Function
  ElseIf Handle <> 0 Then
    TheHandle = Handle
  ElseIf ClassName = "" Then
    TheHandle = FindWindow(vbNullString, Title)
  Else
    TheHandle = FindWindow(ClassName, vbNullString)
  End If
  
  If TheHandle = 0 Then
    Exit Function
  End If
  
  ' Get the coordinates of the window
  ReturnValue = GetWindowRect(TheHandle, rct)
  If ReturnValue = 0 Then  ' Error
    Exit Function
  End If
  
  W_Top = rct.Top
  W_Left = rct.Left
  W_Height = rct.Bottom - rct.Top
  W_Width = rct.Right - rct.Left
  
  W_Top = ((Screen.Height / Screen.TwipsPerPixelY) - W_Height) / 2
  W_Left = ((Screen.Width / Screen.TwipsPerPixelX) - W_Width) / 2
  
  ShowWindow TheHandle, SW_SHOW
  ShowWindow TheHandle, SW_RESTORE
  
  MoveWindow TheHandle, W_Left, W_Top, W_Width, W_Height, True
  
  CenterWindow = True
  
  Exit Function
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  Else                        ' Other Error
   'MsgBox Err.Source & " encountered the following error:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
    CenterWindow = False
    Exit Function
  End If
  
End Function

'====================================================================================
' EndTheProgram
'
' Description:
' ------------
' Unloads all open forms and ends the program.  The ExitingProgram variable is set
' to TRUE to let the rest of the program know that it's trying to end the program.
'
' Param                 Description
' ---------------------------------
' CallEnd               Optional. If set to TRUE, the "End" function is called
'                       to make sure the program is ended.  If the program being
'                       ended contains any kind of sub-classing, this should be
'                       set to FALSE to avoid the program crashing.
'
' Return:
' -------
' Program ends
'
'====================================================================================
Public Sub EndTheProgram(Optional ByVal CallEnd As Boolean = False)
On Error Resume Next
  
  Dim Form As Form
  
  ExitingProgram = True
  
  For Each Form In Forms
    Unload Form
    Set Form = Nothing
  Next
  
  If CallEnd = True Then
    End
  End If
  
End Sub

'====================================================================================
' FileDelete
'
' Description:
' ------------
' Function that will delete a file reguardless of it's attributes.  The Visual
' Basic function "Kill()" will not delete a file if the file's attributes are set
' to System, Hidden, and/or Read-Only
'
' Param                 Description
' ---------------------------------
' FilePath              Specifies the file to delete
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function FileDelete(ByVal FilePath As String) As Boolean
On Error GoTo ErrorTrap
  
  SetFileAttributes FilePath, FILE_ATTRIBUTE_NORMAL
  DoEvents
  Kill FilePath
  
  FileDelete = True
  
  Exit Function
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  ElseIf Err.Number = 53 Then ' File Not Found
    Err.Clear
    Resume Next
  ElseIf Err.Number = 76 Then ' Path Not Found
    Err.Clear
    Resume Next
  Else                        ' Other Error
    MsgBox FilePath & Chr(13) & Chr(13) & Err.Source & " encountered the following error while trying to delete this file:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
    FileDelete = False
    Exit Function
  End If
  
End Function

'====================================================================================
' FormPosition
'
' Description:
' ------------
' Function that sets the specified window to TOP MOST, or NOT.
'
' Param                 Description
' ---------------------------------
' FormHandle            Specifies the window to set to TOP MOST / NOT TOPMOST
' MakeTopMost           TRUE  = Specified window is brought to the top of the
'                               Z Order and stays on top of all other windows
'                               regardless of which window has the focus.
'                       FALSE = Specified window keeps it's current Z Order
'                               and gives / takes focus normally.
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function FormPosition(ByVal FormHandle As Long, ByVal MakeTopMost As Boolean) As Boolean
On Error GoTo ErrorTrap
  
  Dim ReturnValue As Long
  Dim Flag As Long
  
  If MakeTopMost = True Then
    Flag = HWND_TOPMOST
  Else
    Flag = HWND_NOTOPMOST
  End If
  
  ReturnValue = SetWindowPos(FormHandle, Flag, 0, 0, 0, 0, SWP_FLAGS)
  If ReturnValue <> 0 Then
    FormPosition = True
  End If
  
  Exit Function
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  Else                        ' Other Error
   'MsgBox Err.Source & " encountered the following error:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
    FormPosition = False
    Exit Function
  End If
  
End Function

'====================================================================================
' FormPosition
'
' Description:
' ------------
' Perminently removes the "X" close button from the upper right corner of the
' specified window.
'
' NOTE : This should be called in the Form_Load event, or the window should be
'        refreshed after this function is called to ensure proper drawing of the
'        "X" close button on the window.
'
' Param                 Description
' ---------------------------------
' FormHandle            Specifies the window to remove the "X" close button from
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function FormRemoveX(ByVal FormHandle As Long) As Boolean
On Error GoTo ErrorTrap
  
  Dim SystemMenu As Long
  Dim ReturnValue As Long
  
  SystemMenu = GetSystemMenu(FormHandle, 0)
  If SystemMenu = 0 Then
    FormRemoveX = False
    Exit Function
  Else
    ReturnValue = RemoveMenu(SystemMenu, 6, MF_BYPOSITION)
    If ReturnValue = 0 Then
      FormRemoveX = False
      Exit Function
    End If
  End If
  
  FormRemoveX = True
  
  Exit Function
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  Else                        ' Other Error
   'MsgBox Err.Source & " encountered the following error:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
    FormRemoveX = False
    Exit Function
  End If
  
End Function

'====================================================================================
' GetAppVersion
'
' Description:
' ------------
' Function that strips off the extra Zeros from the end of the version number.  If
' the Major Version = 1, the Minor Version = 2000, and the Revision Version = 5600
' then this returns 1.2.56
'
' Param                 Description
' ---------------------------------
' ReturnVersion         Returns the revised version number of the application
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function GetAppVersion(ByRef ReturnVersion As String) As Boolean
On Error Resume Next
  
  Dim MyCounter As Integer
  Dim CharLeft As String
  Dim CharRight As String
  Dim VersionSoFar As String
  Dim NumberToStrip As String
  
  ' Major revision number
  VersionSoFar = CStr(App.Major) & "."
  
  ' Minor revision number
  NumberToStrip = CStr(App.Minor)
  GoSub StripZeros
  VersionSoFar = VersionSoFar & "."
  
  ' Revision number
  NumberToStrip = CStr(App.Revision)
  GoSub StripZeros
  
  ReturnVersion = VersionSoFar
  GetAppVersion = True
  
  Exit Function
  
StripZeros:
  
  If NumberToStrip = "" Then
    Return
  ElseIf NumberToStrip = "0" Then
    VersionSoFar = VersionSoFar & "0"
    Return
  End If
  
  For MyCounter = 1 To Len(NumberToStrip)
    CharRight = Right(NumberToStrip, MyCounter)
    CharLeft = Left(CharRight, 1)
    If CharLeft <> "0" Then
      VersionSoFar = VersionSoFar & Left(NumberToStrip, Len(NumberToStrip) - (MyCounter - 1))
      Return
    End If
  Next
  
  VersionSoFar = VersionSoFar & CharLeft
  Return
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  Else                        ' Other Error
   'MsgBox Err.Source & " encountered the following error:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
    GetAppVersion = False
    Exit Function
  End If
  
End Function

'====================================================================================
' GetWinPath
'
' Description:
' ------------
' Returns the user's Windows install directory.
'
' Param                 Description
' ---------------------------------
' NONE
'
' Return:
' -------
' Succeeded = Returns full 32bit path to the Windows install directory
' Failed    = Returns vbNullString
'
'====================================================================================
Public Function GetWinPath() As String
On Error Resume Next

  Dim strFolder As String
  Dim lngResult As Long
  
  strFolder = String(MAX_PATH, 0)
  lngResult = GetWindowsDirectory(strFolder, MAX_PATH)
  If lngResult <> 0 Then
    GetWinPath = Left(strFolder, InStr(strFolder, Chr(0)) - 1)
  Else
    GetWinPath = ""
  End If
  
End Function

'====================================================================================
' GetWinSysPath
'
' Description:
' ------------
' Windows 95 : Returns the user's Windows\System install directory
' Windows NT : Returns the user's Windows\System32 install directory.
'
' Param                 Description
' ---------------------------------
' NONE
'
' Return:
' -------
' Succeeded = Returns full 32bit path to the Windows\System install directory
' Failed    = Returns vbNullString
'
'====================================================================================
Public Function GetWinSysPath() As String
On Error Resume Next

  Dim strFolder As String
  Dim lngResult As Long
  
  strFolder = String(MAX_PATH, 0)
  lngResult = GetSystemDirectory(strFolder, MAX_PATH)
  
  If lngResult <> 0 Then
    GetWinSysPath = Left(strFolder, InStr(strFolder, Chr(0)) - 1)
  Else
    GetWinSysPath = ""
  End If
  
End Function

'====================================================================================
' GetWinPath
'
' Description:
' ------------
' Returns the user's Windows install directory.
'
' Param                 Description
' ---------------------------------
' NONE
'
' Return:
' -------
' Succeeded = Returns the temporary file path as follows:
'             1. The path specified by the TMP environment variable.
'             2. The path specified by the TEMP environment variable,
'                if TMP is not defined.
'             3. The current directory, if both TMP and TEMP are not defined.
' Failed    = Returns vbNullString
'
'====================================================================================
Public Function GetWinTemp() As String
On Error Resume Next
  
  Dim strFolder As String
  Dim lngResult As Long
  
  strFolder = String(MAX_PATH, 0)
  lngResult = GetTempPath(MAX_PATH, strFolder)
  
  If lngResult <> 0 Then
    GetWinTemp = Left(strFolder, InStr(strFolder, Chr(0)) - 1)
  Else
    GetWinTemp = ""
  End If
  
End Function

'====================================================================================
' GetX
'
' Description:
' ------------
' Returns the current X axis location of the cursor on the screen.  The upper left
' corner of the screen is the start point (0,0).
'
' Param                 Description
' ---------------------------------
' NONE
'
' Return:
' -------
' Succeeded = Returns the current X coordinate of the cursor in pixels
' Failed    = Returns -1
'
'====================================================================================
Public Function GetX() As Long
On Error Resume Next
  
  Dim MyPoint As POINTAPI
  Dim ReturnValue As Long
  
  ReturnValue = GetCursorPos(MyPoint)
  If ReturnValue <> 0 Then
    GetX = MyPoint.X
  Else
    GetX = -1
  End If
  
End Function

'====================================================================================
' GetY
'
' Description:
' ------------
' Returns the current Y axis location of the cursor on the screen.  The upper left
' corner of the screen is the start point (0,0).
'
' Param                 Description
' ---------------------------------
' NONE
'
' Return:
' -------
' Succeeded = Returns the current Y coordinate of the cursor in pixels
' Failed    = Returns -1
'
'====================================================================================
Public Function GetY() As Long
On Error Resume Next
  
  Dim MyPoint As POINTAPI
  Dim ReturnValue As Long
  
  ReturnValue = GetCursorPos(MyPoint)
  If ReturnValue <> 0 Then
    GetY = MyPoint.Y
  Else
    GetY = -1
  End If
  
End Function

'====================================================================================
' INI_Read
'
' Description:
' ------------
' Returns the value of the specified key in the specified section.  If you look at
' the WIN.INI file, it will look something like this :
'
' [Windows]
' Load=
' Run=
' NullPort = None
' MouseTrails = -7
' Device=HP LaserJet 4,HPPCL5MS,\\NTSERVER\HPLaserJ
'
' In this case, "Windows" is the section name, "MouseTrails" is the key name, and the
' return value would be "-7".
'
' Param                 Description
' ---------------------------------
' SectionName           Specifies the section to find the specified key in
' KeyName               Specifies the key to get the value from
' INIPath               Specifies the full path to the .INI file to get the value from
' DefaultValue          Optional. Specifies a default value to return.  If the
'                       specified file does not exist, if the section is does not
'                       exist, if the specified key does not exist, or if the value
'                       of the specified key is vbNullString, then this value is returned
'
' Return:
' -------
' Returns the specified key value, or the specified DefaultValue
'
'====================================================================================
Public Function INI_Read(ByVal SectionName As String, ByVal KeyName As String, ByVal INIPath As String, Optional ByVal DefaultValue As String = "") As String
On Error Resume Next
  
  Dim ln As Long
  Dim ReturnValue As String * MAX_PATH
  
  ln = GetPrivateProfileString(SectionName, KeyName, DefaultValue, ReturnValue, Len(ReturnValue), INIPath)
  INI_Read = Left(ReturnValue, ln)
  
End Function

'====================================================================================
' INI_Write
'
' Description:
' ------------
' Returns the value of the specified key in the specified section.  If you look at
' the WIN.INI file, it will look something like this :
'
' [Windows]
' Load=
' Run=
' NullPort = None
' MouseTrails = -7
' Device=HP LaserJet 4,HPPCL5MS,\\NTSERVER\HPLaserJ
'
' In this case, "Windows" is the section name, "MouseTrails" is the key name, and the
' return value would be "-7".
'
' Param                 Description
' ---------------------------------
' SectionName           Specifies the section to find the specified key in
' KeyName               Specifies the key to write the value to
' Value                 Specifies the value to write to the .INI file
' INIPath               Specifies the full path to the .INI file to write to
'
' Return:
' -------
' TRUE  = Succeeded
' FALSE = Failed
'
'====================================================================================
Public Function INI_Write(ByVal SectionName As String, ByVal KeyName As String, ByVal Value As String, ByVal INIPath As String) As Boolean
On Error Resume Next
  
  Dim ReturnValue As Long
  
  ReturnValue = WritePrivateProfileString(SectionName, KeyName, Value, INIPath)
  If ReturnValue = 0 Then
    INI_Write = False
  Else
    INI_Write = True
  End If
  
End Function
